#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright © 2018, 2019 Endless Mobile, Inc.
# Copyright © 2023 Philip Withnall
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA

"""Integration tests for the gio utility."""

import platform
import subprocess
import sys
import tempfile
import unittest

from pathlib import Path

import taptestrunner
import testprogramrunner


class TestGioTool(testprogramrunner.TestProgramRunner):
    """Integration test for running the gio tool.

    This can be run when installed or uninstalled. When uninstalled, it
    requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

    The idea with this test harness is to test the gio utility, its
    handling of command line arguments, its exit statuses, and its actual
    effects on the file system.
    """

    PROGRAM_NAME = "gio"
    PROGRAM_TYPE = testprogramrunner.ProgramType.NATIVE

    def runGio(self, *args):
        return self.runTestProgram(args)

    def test_help(self):
        """Test the --help argument and help subcommand."""
        result = self.runGio("--help")
        result2 = self.runGio("help")

        self.assertEqual(result.out, result2.out)
        self.assertEqual(result.err, result2.err)

        self.assertIn("Usage:\n  gio COMMAND", result.out)
        self.assertIn("List the contents of locations", result.out)

    def test_no_args(self):
        """Test running with no arguments at all."""
        with self.assertRaises(subprocess.CalledProcessError):
            self.runGio()

    def test_info_non_default_attributes(self):
        """Test running `gio info --attributes` with a non-default list."""
        with tempfile.NamedTemporaryFile(dir=self.tmpdir.name) as tmpfile:
            result = self.runGio(
                "info", "--attributes=standard::content-type", tmpfile.name
            )
            if sys.platform == "darwin":
                self.assertIn("standard::content-type: public.text", result.out)
            else:
                self.assertIn(
                    "standard::content-type: application/x-zerosize", result.out
                )


@unittest.skipIf(platform.system() == "Darwin", "gio launch not supported on darwin")
class TestGioLaunchExpandsDesktopEntry(testprogramrunner.TestProgramRunner):
    """Integration test for `gio launch` with field code %k in the Exec line.

    This can be run when installed or uninstalled. When uninstalled, it
    requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

    The idea with this test harness is to test that the `gio launch` command
    expands the `%k` field code in a desktop entry's Exec line to its location,
    i.e. its absolute path.
    """

    PROGRAM_NAME = "gio"
    PROGRAM_TYPE = testprogramrunner.ProgramType.NATIVE

    TEMPLATE = """
[Desktop Entry]
Type = Application
Name = Test
Exec = {python} -c 'print("%k")'
"""

    def setUp(self):
        super().setUp()

        self.parent = Path(self.tmpdir.name).resolve()

        self.folder = self.parent / "folder"
        self.entry = self.folder / "desktop.entry"

        self.sibling = self.parent / "sibling"

        self.folder.mkdir(exist_ok=True, parents=True)
        self.sibling.mkdir(exist_ok=True, parents=True)

        with self.entry.open("w", encoding="utf-8") as fd:
            fd.write(self.TEMPLATE.format(python=sys.executable))

    def launchAndCheck(self, entry: Path, cwd: Path = None):
        result = self.runTestProgram(["launch", str(entry)], cwd=str(cwd))

        self.assertIn(str(self.entry), result.out)

    def test_absolute_from_folder(self):
        """Test with absolute path, with changing working directory to folder."""
        self.launchAndCheck(self.entry, cwd=self.folder)

    def test_absolute_from_parent(self):
        """Test with absolute path, with changing working directory to parent."""
        self.launchAndCheck(self.entry, cwd=self.parent)

    def test_absolute_from_sibling(self):
        """Test with absolute path, with changing working directory to sibling."""
        self.launchAndCheck(self.entry, cwd=self.sibling)

    def test_relative_from_folder(self):
        """Test with relative path, with changing working directory to folder."""
        self.launchAndCheck(self.entry.relative_to(self.folder), cwd=self.folder)

    def test_relative_from_parent(self):
        """Test with relative path, with changing working directory to parent."""
        self.launchAndCheck(self.entry.relative_to(self.parent), cwd=self.parent)

    def test_relative_from_sibling(self):
        """Test with relative path, with changing working directory to sibling."""
        self.launchAndCheck(
            Path("..") / self.entry.relative_to(self.parent), cwd=self.sibling
        )


if __name__ == "__main__":
    unittest.main(testRunner=taptestrunner.TAPTestRunner())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright © 2026 Philip Withnall
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA

"""Integration tests for the gdbus utility."""

import subprocess
import unittest

import taptestrunner
import testprogramrunner

try:
    # Do all non-standard imports here so we can skip the tests if any
    # needed packages are not available.
    from gi.repository import Gio

    class TestGdbusTool(testprogramrunner.TestProgramRunner):
        """Integration test for running the gdbus tool.

        This can be run when installed or uninstalled. When uninstalled, it
        requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

        The idea with this test harness is to test the gdbus utility, its
        handling of command line arguments, its exit statuses, and its actual
        effects on the bus.
        """

        PROGRAM_NAME = "gdbus"
        PROGRAM_TYPE = testprogramrunner.ProgramType.NATIVE

        def setUp(self):
            self.test_dbus = Gio.TestDBus.new(Gio.TestDBusFlags.NONE)
            self.test_dbus.up()

        def tearDown(self):
            self.test_dbus.stop()
            self.test_dbus.down()

        def runGdbus(self, *args):
            return self.runTestProgram(
                args,
                environment={
                    "DBUS_SESSION_BUS_ADDRESS": self.test_dbus.get_bus_address(),
                    "DBUS_SYSTEM_BUS_ADDRESS": "/dev/null",
                },
            )

        def test_help(self):
            """Test the --help argument and help subcommand."""
            result = self.runGdbus("--help")
            result2 = self.runGdbus("help")

            self.assertEqual(result.out, result2.out)
            self.assertEqual(result.err, result2.err)

            self.assertIn("Usage:\n  gdbus COMMAND", result.out)
            self.assertIn("Invoke a method on a remote object", result.out)

        def test_no_args(self):
            """Test running with no arguments at all."""
            with self.assertRaises(subprocess.CalledProcessError):
                self.runGdbus()

        def test_call_method(self):
            """Test running `gdbus` with a valid method call."""
            result = self.runGdbus(
                "call",
                "--session",
                "--dest",
                "org.freedesktop.DBus",
                "--object-path",
                "/org/freedesktop/DBus",
                "--method",
                "org.freedesktop.DBus.ListNames",
            )
            self.assertIn("'org.freedesktop.DBus'", result.out)

        def test_call_invalid_method(self):
            """Test running `gdbus` with an invalid method name."""
            with self.assertRaises(subprocess.CalledProcessError):
                result = self.runGdbus(
                    "call",
                    "--session",
                    "--dest",
                    "org.freedesktop.DBus",
                    "--object-path",
                    "/org/freedesktop/DBus",
                    "--method",
                    "org.freedesktop.DBus/ListNames",
                )
                self.assertIn(
                    "Error: Method name “org.freedesktop.DBus/ListNames” is invalid",
                    result.err,
                )

except ImportError as e:

    @unittest.skip("Cannot import %s" % e.name)
    class TestGdbusTool(unittest.TestCase):
        def test_call_method(self):
            pass


if __name__ == "__main__":
    unittest.main(testRunner=taptestrunner.TAPTestRunner())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright © 2022 Emmanuel Fleury <emmanuel.fleury@gmail.com>
# Copyright © 2022 Marco Trevisan <mail@3v1n0.net>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA

""" Integration tests for g_message functions on low-memory. """

import os
import unittest

import taptestrunner
import testprogramrunner


class TestMessagesLowMemory(testprogramrunner.TestProgramRunner):
    """Integration test for checking g_message()’s behavior on low memory.

    This can be run when installed or uninstalled. When uninstalled,
    it requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

    The idea with this test harness is to test if g_message and friends
    assert instead of crashing if memory is exhausted, printing the expected
    error message.
    """

    PROGRAM_NAME = "messages-low-memory"
    PROGRAM_TYPE = testprogramrunner.ProgramType.NATIVE

    def test_message_memory_allocation_failure(self):
        """Test running g_message() when memory is exhausted."""
        result = self.runTestProgram([], should_fail=True)

        if result.info.returncode == 77:
            self.skipTest("Not supported")

        if os.name == "nt":
            self.assertEqual(result.info.returncode, 3)
        else:
            self.assertEqual(result.info.returncode, -6)
        self.assertIn("failed to allocate memory", result.err)


if __name__ == "__main__":
    unittest.main(testRunner=taptestrunner.TAPTestRunner())



#!/usr/bin/python3
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA


def init(args, options):
    assert options["version"] == 1


class HeaderCodeGenerator:
    def __init__(self, generator):
        self.ifaces = generator.ifaces
        self.outfile = generator.outfile

        generator.skeleton_type_camel = "NewDBusInterfaceSkeleton"

    def generate_includes(self):
        self.outfile.write("/* codegen-test-extension include */\n")

    def declare_types(self):
        for i in self.ifaces:
            self.outfile.write(
                "/* codegen-test-extension declare type for iface %s */\n" % i.name
            )


class CodeGenerator:
    def __init__(self, generator):
        self.ifaces = generator.ifaces
        self.outfile = generator.outfile

        generator.skeleton_type_upper = "NEW_TYPE_DBUS_INTERFACE_SKELETON"

    def generate_body_preamble(self):
        self.outfile.write("/* codegen-test-extension body preamble */\n")

    def generate(self):
        for i in self.ifaces:
            self.outfile.write(
                "/* codegen-test-extension generate for iface %s */\n" % i.name
            )



import sys
import os
import io
import re


def main(argv):
    # Fix paths in lcov files generated on a Windows host so they match our
    # current source layout.
    paths = argv[1:]

    for path in paths:
        print("cov-fixup:", path)
        text = io.open(path, "r", encoding="utf-8").read()
        text = text.replace("\\\\", "/")
        text = text.replace("\\", "/")
        old_root = re.search(":(.*?)/glib/.*?$", text, re.MULTILINE).group(1)
        new_root = os.path.dirname(os.getcwd())
        if old_root != new_root:
            print("replacing %r with %r" % (old_root, new_root))
        text = text.replace(old_root, new_root)
        with io.open(path, "w", encoding="utf-8") as h:
            h.write(text)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2018 Collabora Inc.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General
# Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
#
# Author: Xavier Claessens <xavier.claessens@collabora.com>

import os
import sys
import tempfile
import subprocess

if "GLIB_TEST_COMPILATION" not in os.environ:
    print(
        """Test disabled because GLIB_TEST_COMPILATION is not set in the env.
If you wish to run this test, set GLIB_TEST_COMPILATION=1 in the env,
and make sure you have glib build dependencies installed, including
meson."""
    )
    sys.exit(77)

if len(sys.argv) != 2:
    print("Usage: {} <gio-2.0.pc dir>".format(os.path.basename(sys.argv[0])))
    sys.exit(1)

test_dir = os.path.dirname(sys.argv[0])

with tempfile.TemporaryDirectory() as builddir:
    env = os.environ.copy()
    env["PKG_CONFIG_PATH"] = sys.argv[1]
    sourcedir = os.path.join(test_dir, "static-link")

    # Ensure we can static link and run a test app
    subprocess.check_call(["meson", "setup", sourcedir, builddir], env=env)
    subprocess.check_call(["meson", "test", "-C", builddir], env=env)
    # FIXME: This probably only works on Linux
    out = subprocess.check_output(
        ["ldd", os.path.join(builddir, "test-static-link")], env=env
    ).decode()
    if "libgio" in out:
        print("test-static-link is dynamically linked on libgio")
        exit(1)



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Marco Trevisan (Treviño)

import argparse
import gitlab
import os
import time
import sys

from datetime import datetime


def has_self_mr_note(mr, current_user_id, contents):
    for note in mr.notes.list(get_all=True):
        if note.author["id"] == current_user_id:
            if contents in note.body:
                return True
    return False


def get_latest_job_for_mr_parent(project, mr, job_name):
    mr_commits = mr.commits(get_all=True)
    first_mr_commit = list(mr_commits)[-1]

    pipelines = []
    job = None

    for parent_sha in first_mr_commit.parent_ids:
        pipelines.extend(
            project.pipelines.list(
                sha=parent_sha,
                ref=mr.target_branch,
                get_all=True,
                order_by="id",
                sort="desc",
            )
        )

    for pipeline in pipelines:
        jobs = pipeline.jobs.list(all=True)

        for job in jobs:
            if job.status == "success" and job.name == job_name:
                return job


if __name__ == "__main__":
    server_uri = os.environ["CI_SERVER_URL"]
    project_path = os.environ["CI_PROJECT_PATH"]
    token_file = os.environ["GIR_CHECK_TOKEN_FILE"]

    parser = argparse.ArgumentParser()
    parser.add_argument("--mr", required=True)
    parser.add_argument("--job", required=True)
    parser.add_argument("--sha")
    parser.add_argument("--job-id-output")
    parser.add_argument("--last-target-job-id-output")
    parser.add_argument("--skip-if-mr-contains-our-comment-text")
    args = parser.parse_args()

    with open(token_file, mode="rt", encoding="utf-8") as tf:
        token = tf.read().strip()

    gl = gitlab.Gitlab(server_uri, private_token=token)
    project = gl.projects.get(project_path)

    [mr] = project.mergerequests.list(iids=[args.mr])

    if args.sha and not mr.sha.startswith(args.sha):
        print(f"MR !{mr.iid} not matching SHA {args.sha}", flush=True)
        sys.exit(77)

    if args.skip_if_mr_contains_our_comment_text:
        print(
            f"Checking comment on MR {server_uri}/{project_path}/-/"
            + f"merge_requests/{mr.iid}",
            flush=True,
        )

        if has_self_mr_note(
            mr,
            gl.http_get("/user")["id"],
            args.skip_if_mr_contains_our_comment_text,
        ):
            print(
                f"MR !{mr.iid} contains already a matching comment, skipping!",
                flush=True,
            )
            sys.exit(77)

    if args.last_target_job_id_output:
        job = get_latest_job_for_mr_parent(project, mr, args.job)

        if job is None:
            print(
                f"Impossible to find a valid job for branch {mr.target_branch}",
                file=sys.stderr,
                flush=True,
            )
            sys.exit(77)

        if job.artifacts_expire_at:
            artifacts_expire_date = datetime.fromisoformat(job.artifacts_expire_at)
            if datetime.now(artifacts_expire_date.tzinfo) >= artifacts_expire_date:
                print(
                    f"Artifacts for job {job.id} have exipred",
                    file=sys.stderr,
                    flush=True,
                )
                sys.exit(77)

        with open(args.last_target_job_id_output, "wt", encoding="utf-8") as f:
            f.write(f"{job.id}\n")

    # Find the project that is associated to the merge request's source branch
    if mr.source_project_id != project.id:
        source_project = gl.projects.get(mr.source_project_id)
    else:
        source_project = project

    try:
        pipelines = source_project.pipelines.list(sha=mr.sha, ref=mr.source_branch)
        ci_pipeline_id = os.environ.get("CI_PIPELINE_ID")
        [pipeline] = [
            p for p in pipelines if p.source != "trigger" and p.id != ci_pipeline_id
        ]
    except ValueError:
        print(
            f"No unique pipeline found for {server_uri}/{source_project.path_with_namespace}/-/"
            + f"commit/{mr.sha}",
            flush=True,
        )
        sys.exit(1)

    print(
        f"Found matching pipeline {server_uri}/{source_project.path_with_namespace}/-/"
        + f"pipelines/{pipeline.id}",
        flush=True,
    )

    try:
        [job] = [
            j
            for j in pipeline.jobs.list(all=True)
            if j.name == args.job and j.commit["id"] == mr.sha
        ]
    except ValueError:
        print(
            f"No unique '{args.job}' job found for {server_uri}/{source_project.path_with_namespace}"
            + f"/-/commit/{mr.sha}",
            flush=True,
        )
        sys.exit(1)

    print(
        f"Found matching job {server_uri}/{source_project.path_with_namespace}/-/jobs/{job.id}",
        flush=True,
    )

    if args.job_id_output:
        with open(args.job_id_output, "wt", encoding="utf-8") as f:
            f.write(f"{job.id}\n")

    waiting = False
    while job.status != "success":
        if job.status in (
            "pending",
            "running",
            "created",
            "preparing",
            "waiting_for_resource",
        ):
            if not waiting:
                print("Waiting for the job to complete...", flush=True)
                job = source_project.jobs.get(job.id)
                waiting = True

            time.sleep(5)
            job.refresh()
            continue

        if job.status == "canceled":
            print(f"Job {job.id} was cancelled", file=sys.stderr, flush=True)
            sys.exit(77)

        print(f"Job {job.id} did not succeed", file=sys.stderr, flush=True)
        sys.exit(1)

    print(f"Job {job.id} completed!", flush=True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright © 2022 Emmanuel Fleury <emmanuel.fleury@gmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA

""" Integration tests for g_assert() functions. """

import os
import shutil
import tempfile
import unittest

import taptestrunner
import testprogramrunner

GDB_SCRIPT = """
# Work around https://sourceware.org/bugzilla/show_bug.cgi?id=22501
set confirm off
set print elements 0
set auto-load safe-path /
run
print *((char**) &__glib_assert_msg)
quit
"""


class TestAssertMessage(testprogramrunner.TestProgramRunner):
    """Integration test for throwing message on g_assert().

    This can be run when installed or uninstalled. When uninstalled,
    it requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

    The idea with this test harness is to test if g_assert() prints
    an error message when called, and that it saves this error
    message in a global variable accessible to gdb, so that developers
    and automated tools can more easily debug assertion failures.
    """

    PROGRAM_NAME = "assert-msg-test"
    PROGRAM_TYPE = testprogramrunner.ProgramType.NATIVE

    def setUp(self):
        super().setUp()
        self.__gdb = shutil.which("gdb")

    def runAssertMessage(self, *args):
        return self.runTestProgram(args, should_fail=True)

    def runGdbAssertMessage(self, *args):
        if self.__gdb is None:
            return testprogramrunner.Result()

        return self.runTestProgram(args, wrapper_args=["gdb", "-n", "--batch"])

    def test_gassert(self):
        """Test running g_assert() and fail the program."""
        result = self.runAssertMessage()

        if os.name == "nt":
            self.assertEqual(result.info.returncode, 3)
        else:
            self.assertEqual(result.info.returncode, -6)
        self.assertIn("assertion failed: (42 < 0)", result.out)

    def test_gdb_gassert(self):
        """Test running g_assert() within gdb and fail the program."""
        if self.__gdb is None:
            self.skipTest("GDB is not installed, skipping this test!")
        if {"thread", "address"} & set(
            os.getenv("_GLIB_TEST_SANITIZERS", "").split(",")
        ):
            self.skipTest("GDB can't run under sanitizers")

        with tempfile.NamedTemporaryFile(
            prefix="assert-msg-test-", suffix=".gdb", mode="w", delete=False
        ) as tmp:
            try:
                tmp.write(GDB_SCRIPT)
                tmp.close()
                result = self.runGdbAssertMessage("-x", tmp.name)
            finally:
                os.unlink(tmp.name)

            # Some CI environments disable ptrace (as they’re running in a
            # container). If so, skip the test as there’s nothing we can do.
            if result.info.returncode != 0 and (
                "ptrace: Operation not permitted" in result.err
                or "warning: opening /proc/PID/mem file for lwp" in result.err
            ):
                self.skipTest("GDB is not functional due to ptrace being disabled")

            self.assertEqual(result.info.returncode, 0)
            self.assertIn("$1 = 0x", result.out)
            self.assertIn("assertion failed: (42 < 0)", result.out)


if __name__ == "__main__":
    unittest.main(testRunner=taptestrunner.TAPTestRunner())



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Marco Trevisan (Treviño)

import argparse
import os
import gitlab
import subprocess
import sys


def run_diff(baseline_path, current_path):
    cmd = [
        "diff",
        "-u",
        '--ignore-matching-lines=line="[0-9]\\+"',
        baseline_path,
        current_path,
    ]

    result = subprocess.run(
        cmd,
        capture_output=True,
        text=True,
        timeout=60,
        check=False,
    )

    return result


def diffstat(diff):
    stats = {}  # path -> [add, del]
    current = None

    for line in diff.splitlines():
        if line.startswith("+++ "):
            path = line[4:].strip()
            if path == "/dev/null":
                current = None
                continue
            [current, _] = path.split("\t", 1)
            stats.setdefault(current, [0, 0])
            continue
        if (
            line.startswith("--- ")
            or line.startswith("@@ ")
            or line.startswith("diff ")
            or line.startswith("Index: ")
        ):
            continue
        if current is None:
            continue
        if line.startswith("+") and not line.startswith("+++"):
            stats[current][0] += 1
        elif line.startswith("-") and not line.startswith("---"):
            stats[current][1] += 1

    total_add = sum(v[0] for v in stats.values())
    total_del = sum(v[1] for v in stats.values())

    files_changed = len(stats)
    summary = (
        f"{files_changed} file changed, "
        + f"{total_add} insertions(+), {total_del} deletions(-)"
    )

    max_file_len = max((len(f) for f in stats.keys()), default=0)
    max_add_w = max((len(str(v[0])) for v in stats.values()), default=1)
    max_del_w = max((len(str(v[1])) for v in stats.values()), default=1)

    lines = []
    max_bar_len = 20
    for path, (a, d) in stats.items():
        if (a + d) > max_bar_len:
            add_ratio = a / (a + d)
            a = int(add_ratio * max_bar_len)
            d = int((1 - add_ratio) * max_bar_len)

        bar = "+" * a + "-" * d
        lines.append(
            f"{path.ljust(max_file_len)} | "
            + f"+{a:<{max_add_w}} -{d:<{max_del_w}} {bar}"
        )

    return summary, lines


if __name__ == "__main__":
    server_uri = os.environ["CI_SERVER_URL"]
    project_path = os.environ["CI_PROJECT_PATH"]
    current_job = os.environ["CI_JOB_ID"]
    token_file = os.environ["GIR_CHECK_TOKEN_FILE"]

    parser = argparse.ArgumentParser()
    parser.add_argument("baseline_path", help="Path containing baseline gir files")
    parser.add_argument("current_path", help="Path containing current gir files")
    parser.add_argument("--mr", default=os.environ.get("CI_MERGE_REQUEST_IID"))
    parser.add_argument("--job", default=os.environ.get("CI_JOB_NAME"))
    parser.add_argument("--sha", default=os.environ.get("CI_COMMIT_SHA"))
    parser.add_argument("--comment-stamp", default="")
    args = parser.parse_args()

    if not os.path.exists(args.baseline_path):
        print(f"{args.baseline_path} does not exist, skipping check")
        sys.exit(77)

    res = run_diff(args.baseline_path, args.current_path)

    if res.returncode == 0:
        print("No diff found")
        sys.exit(0)

    if res.returncode != 1:
        print("diff failed")
        if res.stdout:
            print(res.stdout)
        if res.stderr:
            print(res.stderr)
        sys.exit(res.returncode)

    if res.stderr:
        print(f"stderr: {res.stderr}")

    diff = res.stdout
    print(f"{args.baseline_path} and ${args.current_path} files do not match")
    print(diff)
    print("=======================")

    with open(token_file, mode="rt", encoding="utf-8") as tf:
        token = tf.read().strip()

    gl = gitlab.Gitlab(server_uri, private_token=token)
    current_user_id = gl.http_get("/user")["id"]

    project = gl.projects.get(project_path)

    [mr] = project.mergerequests.list(iids=[args.mr])

    comment_id = f"comment-{current_user_id}-{args.job}-{args.mr}"

    diff_note = None
    for note in mr.notes.list(get_all=True):
        if note.author["id"] == current_user_id and comment_id in note.body:
            diff_note = note
            break

    diff_summary, diff_stat_lines = diffstat(diff)

    # TODO: Move this inside the f-string when we've a newer enough python
    # that does not fail with
    # E999 SyntaxError: f-string expression part cannot include a backslash
    diff_stat = "\n".join(diff_stat_lines)

    print(diff_summary)
    print(diff_stat)

    body = f"""### Diff found in generated `*.gir` files for `{args.job}`

When comparing generated GIR files in commit {mr.sha}
with the ones in `{mr.target_branch}` branch:

{diff_summary}

```
{diff_stat}
```

<details>
<summary>GIR files diff</summary>

```diff
{diff}
```

`comment-stamp: {args.comment_stamp}`

`comment-id: {comment_id}`

Generated by job [`#{current_job}`]({server_uri}/{project_path}/-/jobs/{current_job})

</details>
"""

    contents = {"body": body}
    if diff_note:
        mr.notes.update(diff_note.id, contents)
    else:
        diff_note = mr.notes.create(contents)

    print(
        f"MR Comment updated: {server_uri}/{project_path}/-/"
        + f"merge_requests/{mr.iid}/#note_{diff_note.id}"
    )



#!/usr/bin/env python3
#
# Copyright © 2022-2024 Collabora, Ltd.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Original author: Xavier Claessens

"""
This script checks Meson configuration logs to verify no installed file is
missing installation tag.
"""

import argparse
import json
from pathlib import Path


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("builddir", type=Path, nargs="?", default=".")
    args = parser.parse_args()

    print("# TAP version 13")

    count = 0
    bad = 0
    path = args.builddir / "meson-info" / "intro-install_plan.json"
    with path.open(encoding="utf-8") as f:
        install_plan = json.load(f)
        for target in install_plan.values():
            for info in target.values():
                count += 1

                if not info["tag"]:
                    bad += 1
                    dest = info["destination"]
                    print(f"not ok {bad} - Missing install_tag for {dest}")

    if bad == 0:
        print(f"ok 1 - All {count} installed files have install_tag")
        print("1..1")
        return 0
    else:
        print(f"# {bad}/{count} installed files do not have install_tag")
        print(f"1..{bad}")
        return 1


if __name__ == "__main__":
    exit(main())



#!/usr/bin/env python3
#
# Copyright © 2022 Collabora Inc.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Original author: Xavier Claessens <xclaesse@gmail.com>

import argparse
import textwrap
from pathlib import Path


# Disable line length warnings as wrapping the C code templates would be hard
# flake8: noqa: E501


def gen_versions_macros(args, current_minor_version):
    with args.out_path.open("w", encoding="utf-8") as ofile, args.in_path.open(
        "r", encoding="utf-8"
    ) as ifile:
        for line in ifile.readlines():
            if "@GLIB_VERSIONS@" in line:
                for minor in range(2, current_minor_version + 2, 2):
                    ofile.write(
                        textwrap.dedent(
                            f"""\
                        /**
                        * GLIB_VERSION_2_{minor}:
                        *
                        * A macro that evaluates to the 2.{minor} version of GLib, in a format
                        * that can be used by the C pre-processor.
                        *
                        * Since: 2.{max(minor, 32)}
                        */
                        #define GLIB_VERSION_2_{minor}       (G_ENCODE_VERSION (2, {minor}))
                        """
                        )
                    )
            else:
                ofile.write(line)


def gen_doc_sections(args, current_minor_version):
    with args.out_path.open("w", encoding="utf-8") as ofile, args.in_path.open(
        "r", encoding="utf-8"
    ) as ifile:
        for line in ifile.readlines():
            if "@GLIB_VERSIONS@" in line:
                for minor in range(2, current_minor_version + 2, 2):
                    ofile.write(
                        textwrap.dedent(
                            f"""\
                        GLIB_VERSION_2_{minor}
                        """
                        )
                    )
            else:
                ofile.write(line)


def gen_visibility_macros(args, current_minor_version):
    """
    Generates a set of macros for each minor stable version of GLib

    - GLIB_VAR

    - GLIB_DEPRECATED
    - GLIB_DEPRECATED_IN_…
    - GLIB_DEPRECATED_MACRO_IN_…
    - GLIB_DEPRECATED_ENUMERATOR_IN_…
    - GLIB_DEPRECATED_TYPE_IN_…

    - GLIB_AVAILABLE_IN_ALL
    - GLIB_AVAILABLE_IN_…
    - GLIB_AVAILABLE_STATIC_INLINE_IN_…
    - GLIB_AVAILABLE_MACRO_IN_…
    - GLIB_AVAILABLE_ENUMERATOR_IN_…
    - GLIB_AVAILABLE_TYPE_IN_…

    - GLIB_UNAVAILABLE(maj,min)
    - GLIB_UNAVAILABLE_STATIC_INLINE(maj,min)

    The GLIB namespace can be replaced with one of GOBJECT, GIO, GMODULE, GI.
    """

    ns = args.namespace
    with args.out_path.open("w", encoding="utf-8") as f:
        f.write(
            textwrap.dedent(
                f"""\
            #pragma once

            #if (defined(_WIN32) || defined(__CYGWIN__)) && !defined({ns}_STATIC_COMPILATION)
            #  define _{ns}_EXPORT __declspec(dllexport)
            #  define _{ns}_IMPORT __declspec(dllimport)
            #elif __GNUC__ >= 4
            #  define _{ns}_EXPORT __attribute__((visibility("default")))
            #  define _{ns}_IMPORT
            #else
            #  define _{ns}_EXPORT
            #  define _{ns}_IMPORT
            #endif
            #ifdef {ns}_COMPILATION
            #  define _{ns}_API _{ns}_EXPORT
            #else
            #  define _{ns}_API _{ns}_IMPORT
            #endif

            #define _{ns}_EXTERN _{ns}_API extern

            #define {ns}_VAR _{ns}_EXTERN
            #define {ns}_AVAILABLE_IN_ALL _{ns}_EXTERN

            #ifdef GLIB_DISABLE_DEPRECATION_WARNINGS
            #define {ns}_DEPRECATED _{ns}_EXTERN
            #define {ns}_DEPRECATED_FOR(f) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE(maj,min) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE_STATIC_INLINE(maj,min)
            #else
            #define {ns}_DEPRECATED G_DEPRECATED _{ns}_EXTERN
            #define {ns}_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE(maj,min) G_UNAVAILABLE(maj,min) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE_STATIC_INLINE(maj,min) G_UNAVAILABLE(maj,min)
            #endif
            """
            )
        )
        for minor in range(26, current_minor_version + 2, 2):
            f.write(
                textwrap.dedent(
                    f"""
                #if GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_{minor}
                #define {ns}_DEPRECATED_IN_2_{minor} {ns}_DEPRECATED
                #define {ns}_DEPRECATED_IN_2_{minor}_FOR(f) {ns}_DEPRECATED_FOR (f)
                #define {ns}_DEPRECATED_MACRO_IN_2_{minor} GLIB_DEPRECATED_MACRO
                #define {ns}_DEPRECATED_MACRO_IN_2_{minor}_FOR(f) GLIB_DEPRECATED_MACRO_FOR (f)
                #define {ns}_DEPRECATED_ENUMERATOR_IN_2_{minor} GLIB_DEPRECATED_ENUMERATOR
                #define {ns}_DEPRECATED_ENUMERATOR_IN_2_{minor}_FOR(f) GLIB_DEPRECATED_ENUMERATOR_FOR (f)
                #define {ns}_DEPRECATED_TYPE_IN_2_{minor} GLIB_DEPRECATED_TYPE
                #define {ns}_DEPRECATED_TYPE_IN_2_{minor}_FOR(f) GLIB_DEPRECATED_TYPE_FOR (f)
                #else
                #define {ns}_DEPRECATED_IN_2_{minor} _{ns}_EXTERN
                #define {ns}_DEPRECATED_IN_2_{minor}_FOR(f) _{ns}_EXTERN
                #define {ns}_DEPRECATED_MACRO_IN_2_{minor}
                #define {ns}_DEPRECATED_MACRO_IN_2_{minor}_FOR(f)
                #define {ns}_DEPRECATED_ENUMERATOR_IN_2_{minor}
                #define {ns}_DEPRECATED_ENUMERATOR_IN_2_{minor}_FOR(f)
                #define {ns}_DEPRECATED_TYPE_IN_2_{minor}
                #define {ns}_DEPRECATED_TYPE_IN_2_{minor}_FOR(f)
                #endif

                #if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_{minor}
                #define {ns}_AVAILABLE_IN_2_{minor} {ns}_UNAVAILABLE (2, {minor})
                #define {ns}_AVAILABLE_STATIC_INLINE_IN_2_{minor} GLIB_UNAVAILABLE_STATIC_INLINE (2, {minor})
                #define {ns}_AVAILABLE_MACRO_IN_2_{minor} GLIB_UNAVAILABLE_MACRO (2, {minor})
                #define {ns}_AVAILABLE_ENUMERATOR_IN_2_{minor} GLIB_UNAVAILABLE_ENUMERATOR (2, {minor})
                #define {ns}_AVAILABLE_TYPE_IN_2_{minor} GLIB_UNAVAILABLE_TYPE (2, {minor})
                #else
                #define {ns}_AVAILABLE_IN_2_{minor} _{ns}_EXTERN
                #define {ns}_AVAILABLE_STATIC_INLINE_IN_2_{minor}
                #define {ns}_AVAILABLE_MACRO_IN_2_{minor}
                #define {ns}_AVAILABLE_ENUMERATOR_IN_2_{minor}
                #define {ns}_AVAILABLE_TYPE_IN_2_{minor}
                #endif
                """
                )
            )


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("glib_version", help="Current GLib version")
    subparsers = parser.add_subparsers()

    versions_parser = subparsers.add_parser(
        "versions-macros", help="Generate versions macros"
    )
    versions_parser.add_argument("in_path", help="input file", type=Path)
    versions_parser.add_argument("out_path", help="output file", type=Path)
    versions_parser.set_defaults(func=gen_versions_macros)

    doc_parser = subparsers.add_parser(
        "doc-sections", help="Generate glib-sections.txt"
    )
    doc_parser.add_argument("in_path", help="input file", type=Path)
    doc_parser.add_argument("out_path", help="output file", type=Path)
    doc_parser.set_defaults(func=gen_doc_sections)

    visibility_parser = subparsers.add_parser(
        "visibility-macros", help="Generate visibility macros"
    )
    visibility_parser.add_argument("namespace", help="Macro namespace")
    visibility_parser.add_argument("out_path", help="output file", type=Path)
    visibility_parser.set_defaults(func=gen_visibility_macros)

    args = parser.parse_args()
    version = [int(i) for i in args.glib_version.split(".")]
    assert version[0] == 2
    args.func(args, version[1])


if __name__ == "__main__":
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright © 2018 Endless Mobile, Inc.
# Copyright © 2025 Canonical Ltd.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA  02110-1301  USA

"""Integration tests for GLib utilities."""

import os
import shutil
import subprocess
import tempfile
import unittest
import sys

from dataclasses import dataclass
from enum import Enum


class ProgramType(Enum):
    """Enum to define the kind of tool to use"""

    NATIVE = 1
    INTERPRETED = 2


@dataclass
class Result:
    """Class for keeping track of the executable result."""

    info: subprocess.CompletedProcess
    out: str
    err: str
    subs: dict


class TestProgramRunner(unittest.TestCase):
    """Integration test for running glib-based tools.

    This can be run when installed or uninstalled. When uninstalled, it
    requires G_TEST_BUILDDIR or _G_TEST_PROGRAM_RUNNER_PATH to be set.
    """

    PROGRAM_NAME: str = None
    PROGRAM_TYPE: ProgramType = ProgramType.NATIVE
    INTERPRETER: str = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.assertTrue(cls.PROGRAM_NAME, "class PROGRAM_NAME must be set")

        ext = ""
        if cls.PROGRAM_TYPE == ProgramType.NATIVE and os.name == "nt":
            ext = ".exe"

        cls._program_name = f"{cls.PROGRAM_NAME}{ext}"

        if "_G_TEST_PROGRAM_RUNNER_PATH" in os.environ:
            cls.__program = os.path.join(
                os.environ["_G_TEST_PROGRAM_RUNNER_PATH"], cls._program_name
            )
        elif "G_TEST_BUILDDIR" in os.environ:
            cls.__program = os.path.join(
                os.environ["G_TEST_BUILDDIR"], cls._program_name
            )
        else:
            cls.__program = os.path.join(os.path.dirname(__file__), cls._program_name)
            if not os.path.exists(cls.__program):
                cls.__program = shutil.which(cls._program_name)

    def setUp(self):
        print(f"{self.PROGRAM_NAME}: {self.__program}")
        self.assertTrue(os.path.exists(self.__program))

        self.tmpdir = tempfile.TemporaryDirectory()
        self.addCleanup(self.tmpdir.cleanup)
        old_cwd = os.getcwd()
        self.addCleanup(os.chdir, old_cwd)
        os.chdir(self.tmpdir.name)
        print("tmpdir:", self.tmpdir.name)

    def runTestProgram(
        self,
        *args,
        should_fail=False,
        wrapper_args=[],
        environment={},
        cwd=None,
    ) -> Result:
        argv = [self.__program]

        argv.extend(*args)

        # shebang lines are not supported on native
        # Windows consoles
        if self.PROGRAM_TYPE == ProgramType.INTERPRETED and os.name == "nt":
            argv.insert(0, self.INTERPRETER if self.INTERPRETER else sys.executable)

        argv = wrapper_args + argv

        env = os.environ.copy()
        env["LC_ALL"] = "C.UTF-8"
        env["G_DEBUG"] = "fatal-warnings"
        env.update(environment)

        print("Running:", argv)

        if cwd is not None:
            print("Working Directory:", cwd)

        # We want to ensure consistent line endings...
        info = subprocess.run(
            argv,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            env=env,
            universal_newlines=True,
            text=True,
            encoding="utf-8",
            check=False,
            cwd=cwd,
        )

        result = Result(
            info=info,
            out=info.stdout.strip(),
            err=info.stderr.strip(),
            subs=self._getSubs(),
        )

        print("Return code:", result.info.returncode)
        print("Output:\n", result.out)
        print("Error:\n", result.err)

        if should_fail:
            with self.assertRaises(subprocess.CalledProcessError):
                info.check_returncode()
        else:
            info.check_returncode()

        return result

    def _getSubs(self) -> dict:
        return {}
